@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+# cython: language_level=3
 
-## Copyright 2005-2011 by LivingLogic AG, Bayreuth/Germany.
-## Copyright 2005-2011 by Walter Dörwald
+## Copyright 2005-2014 by LivingLogic AG, Bayreuth/Germany.
+## Copyright 2005-2014 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
-## See orasql/__init__.py for the license
+## See ll/xist/__init__.py for the license
 
 
 """
@@ -54,8 +55,13 @@ Options
 		the original one. The default is to grant all privileges to the original
 		grantee.
 
-	``-e``, ``--encoding`` : encoding
-		The encoding of the output (if ``-x`` is not given; default is ``utf-8``).
+	``--include`` : regexp
+		Only include objects in the output if their name contains the regular
+		expression.
+
+	``--exclude`` : regexp
+		Exclude objects from the output if their name contains the regular
+		expression.
 
 
 Example
@@ -68,7 +74,7 @@ Grant all privileges that ``alice`` has in the schema ``user@db`` to ``bob`` in
 """
 
 
-import sys, os, argparse
+import sys, os, re, argparse
 
 from ll import misc, astyle, orasql
 
@@ -91,7 +97,8 @@ def main(args=None):
 	p.add_argument("-k", "--keepjunk", dest="keepjunk", help="Output objects with '$' or 'SYS_EXPORT_SCHEMA_' in their name? (default %(default)s)", default=False, action="store_true")
 	p.add_argument("-i", "--ignore", dest="ignore", help="Ignore errors? (default %(default)s)", default=False, action=misc.FlagAction)
 	p.add_argument("-m", "--mapgrantee", dest="mapgrantee", help="Map grantees (Python expression: list or dict)", default="True")
-	p.add_argument("-e", "--encoding", dest="encoding", help="Encoding for output (default %(default)s)", default="utf-8")
+	p.add_argument(      "--include", dest="include", metavar="REGEXP", help="Include only objects whose name contains PATTERN (default: %(default)s)", type=re.compile)
+	p.add_argument(      "--exclude", dest="exclude", metavar="REGEXP", help="Exclude objects whose name contains PATTERN (default: %(default)s)", type=re.compile)
 
 	args = p.parse_args(args)
 
@@ -120,9 +127,11 @@ def main(args=None):
 	mapgrantee = eval(args.mapgrantee)
 
 	def keep(obj):
-		if args.keepjunk:
-			return True
-		if "$" in obj.name or "/" in obj.name or obj.name.startswith("SYS_EXPORT_SCHEMA_"):
+		if ("$" in obj.name or "/" in obj.name or obj.name.startswith("SYS_EXPORT_SCHEMA_")) and not args.keepjunk:
+			return False
+		if args.include is not None and args.include.search(obj.name) is None:
+			return False
+		if args.exclude is not None and args.exclude.search(obj.name) is not None:
 			return False
 		return True
 
@@ -144,12 +153,12 @@ def main(args=None):
 				if args.execute:
 					try:
 						cursor2.execute(ddl)
-					except orasql.DatabaseError, exc:
+					except orasql.DatabaseError as exc:
 						if not args.ignore or "ORA-01013" in str(exc):
 							raise
 						stderr.writeln("oragrant.py: ", s4error("{}: {}".format(exc.__class__.__name__, str(exc).strip())))
 				else:
-					stdout.writeln(ddl.encode(args.encoding))
+					stdout.writeln(ddl)
 					stdout.writeln()
 
 
